
/*
 * App Weather - A Weather Source API
 *
 * App/Webpage optimized weather data for latitude/longitude points, ZIP/Postal Codes, and OnPoint™ points. The Weather Source App Weather API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate use. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  This API returns current conditions (Nowcast) and a 7-day of daily and hourly forecast. The nowcast and forecasts include limited astronomical data.  The weather icon classes returned by this API correspond to classes used by the open source licensed [Weather Icons](https://developer.weathersourceapis.com/apis/appwx-weather-icons/) font set.      **IMPORTANT:** To authenticate to the Weather Source App Weather API, each request must contain your API key set to a custom header named `X-API-KEY`. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key `C0W60UOFRML47ytbXk4xlLBfv` to access example locations found in this documentation.      When using the \\\"Try it out\\\" capability in the documentation, the API key should be added by clicking the green \\\"Authorize\\\" button at the top of the page. 
 *
 * API version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */
package swagger

import (
	"context"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"fmt"
	"github.com/antihax/optional"
)

// Linger please
var (
	_ context.Context
)

type PostcodesApiService service
/*
PostcodesApiService App Weather response for a postcode.
Returns select nowcast and forecast (7-day - daily and hourly) data for a postcode.  The weather icon classes returned by this API correspond to classes used by the open source licensed [Weather Icons](https://developer.weathersourceapis.com/apis/appwx-weather-icons/) font set.  **IMPORTANT:** To authenticate to the Weather Source App Weather API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param postcode A postcode string.
 * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
 * @param services A comma separated list of service names to return.  **Limiting the query to needed services will improve the query response time.** 
 * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.** 
 * @param optional nil or *PostcodesApiPostcodesPostcodecountryCodeGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
     * @param "Language" (optional.String) -  The language for translatable returned values.&lt;br&gt;
@return AppwxPostcodeObj
*/

type PostcodesApiPostcodesPostcodecountryCodeGetOpts struct {
    UnitScale optional.String
    Language optional.String
}

func (a *PostcodesApiService) PostcodesPostcodecountryCodeGet(ctx context.Context, postcode string, countryCode string, services []string, fields []string, localVarOptionals *PostcodesApiPostcodesPostcodecountryCodeGetOpts) (AppwxPostcodeObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue AppwxPostcodeObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/postcodes/{postcode},{countryCode}"
	localVarPath = strings.Replace(localVarPath, "{"+"postcode"+"}", fmt.Sprintf("%v", postcode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"countryCode"+"}", fmt.Sprintf("%v", countryCode), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}

	localVarQueryParams.Add("services", parameterToString(services, "csv"))
	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	if localVarOptionals != nil && localVarOptionals.Language.IsSet() {
		localVarQueryParams.Add("language", parameterToString(localVarOptionals.Language.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v AppwxPostcodeObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
